ad_page_contract {
    Index page for a workflow.

    @author Lars Pind (lars@pinds.com)
    @creation-date November 20, 2000
    @cvs-id $Id: workflow.tcl,v 1.1 2001/03/13 22:59:27 ben Exp $
} {
    workflow_key:notnull
} -properties {
    workflow:onerow
    context_bar
    workflow_img_tag
    edit_process_url
    export_process_url
    edit_attributes_url
    copy_process_url
} -validate {
    workflow_exists -requires {workflow_key} {
	if { 0 == [db_string workflow_exists "
	select count(*) from wf_workflows 
	where workflow_key = :workflow_key"] } {
	    ad_complain "You seem to have specified a nonexistent workflow."
	}
    }
}

db_1row workflow {
    select w.workflow_key, 
           t.pretty_name,
           w.description,
           count(c.case_id) as num_cases,
           0 as num_unassigned_tasks
    from   wf_workflows w, 
           acs_object_types t, 
           wf_cases c
    where  w.workflow_key = :workflow_key 
    and    w.workflow_key = t.object_type
    and    c.workflow_key (+) = w.workflow_key
    group  by w.workflow_key, t.pretty_name, w.description
} -column_array workflow

set workflow(num_unassigned_tasks) [db_string num_unassigned_tasks {
    select count(*) 
    from   wf_tasks t,
           wf_cases c
    where  t.workflow_key = :workflow_key
    and    t.state = 'enabled'
    and    c.case_id = t.case_id
    and    c.state = 'active'
    and    not exists (select 1 from wf_task_assignments ta where ta.task_id = t.task_id)
}]

set workflow(num_active_cases) [db_string num_active_cases {
    select count(*) 
    from   wf_cases c
    where  c.workflow_key = :workflow_key
    and    c.state = 'active'
}]

set context_bar [ad_context_bar "$workflow(pretty_name)"]


# GIF

set workflow_info [wf_get_workflow_net $workflow_key]
array set workflow $workflow_info
db_release_unused_handles

wf_decorate_workflow workflow

set dot_text [wf_generate_dot_representation workflow]

set tmpfile [wf_graphviz_dot_exec -to_file -output gif $dot_text]
	
set width_and_height ""
if { ![catch { set image_size [ns_gifsize $tmpfile] } error] } {
    if { ![empty_string_p $image_size] } {
	set width_and_height "width=[lindex $image_size 0] height=[lindex $image_size 1]"
    }
}

ad_set_client_property wf wf_net_tmpfile $tmpfile

set workflow_img_tag "<img src=\"workflow-gif?[export_url_vars tmpfile]\" border=0 $width_and_height alt=\"Graphical representation of the process network\">"


if { $workflow(num_cases) > 0 } { 
    set edit_process_url ""
    set edit_attributes_url ""
    set copy_process_url "workflow-copy?[export_vars -url {workflow_key}]"
} else {
    set edit_process_url "define?[export_vars -url {workflow_key}]"
    set edit_attributes_url "attributes?[export_vars -url {workflow_key}]"
    set copy_process_url ""
}

set export_process_url "export?[export_vars -url {workflow_key}]"


ad_return_template





