ad_page_contract {
    Displays information about a case.
    
    @author Lars Pind (lars@pinds.com)
    @creation-date 18 August 2000
    @cvs-id $Id: case.tcl,v 1.1 2001/03/13 22:59:27 ben Exp $
} {
    case_id:integer,notnull
} -properties {
    case:onerow
    context_bar
    actions:multirow
    workflow_img_tag
}

db_1row case_info {
    select case_id, 
           acs_object.name(object_id) as object_name, 
           state,
           workflow_key
    from   wf_cases
    where  case_id = :case_id
} -column_array case
set case(debug_url) "admin/case-debug?[export_vars -url {case_id}]"

set workflow_key $case(workflow_key)

set context_bar [ad_context_bar "$case(object_name) case"]

template::multirow create actions url title
switch $case(state) {
    active {
	template::multirow append actions "case-state-change?[export_url_vars case_id]&action=suspend" "suspend"
	template::multirow append actions "case-state-change?[export_url_vars case_id]&action=cancel" "cancel"
    }
    suspended {
	template::multirow append actions "case-state-change?[export_url_vars case_id]&action=resume" "resume"
	template::multirow append actions "case-state-change?[export_url_vars case_id]&action=cancel" "cancel"
    }
    canceled {
	template::multirow append actions "case-state-change?[export_url_vars case_id]&action=resume" "resume"
    }
}

set workflow_info [wf_get_workflow_net $case(workflow_key)]
array set workflow $workflow_info
wf_decorate_workflow workflow

set date_format "Mon fmDDfm, YYYY HH24:MI:SS"

db_multirow current_tasks current_tasks {
    select t.task_id, 
           t.transition_key, 
           t.state, 
           t.case_id,
           tr.transition_name,
           to_char(t.enabled_date, :date_format) as enabled_date_pretty
    from   wf_tasks t, wf_transitions tr
    where  t.case_id = :case_id
    and    t.state in ('enabled', 'started')
    and    tr.workflow_key = t.workflow_key
    and    tr.transition_key = t.transition_key
    order by t.enabled_date desc
}

db_multirow old_tasks old_tasks {
    select t.task_id, 
           t.transition_key, 
           t.state, 
           t.case_id,
           tr.transition_name,
           to_char(t.enabled_date, :date_format) as enabled_date_pretty
    from   wf_tasks t, wf_transitions tr
    where  t.case_id = :case_id
    and    t.state not in ('enabled', 'started')
    and    tr.workflow_key = t.workflow_key
    and    tr.transition_key = t.transition_key
    order by t.enabled_date desc
}


#####
#
# Add marking to the graph
#
#####

foreach place_key $workflow(places) {
    set workflow(place,$place_key,num_tokens) 0
}
foreach transition_key $workflow(transitions) {
    set workflow(transition,$transition_key,num_tokens) 0
}

db_foreach tokens {
    select tok.token_id, 
           tok.place_key,
           tok.locked_task_id,
           ta.transition_key
    from   wf_tokens tok,
           wf_tasks ta
    where  tok.case_id = :case_id
    and    ta.task_id (+) = tok.locked_task_id
    and    tok.state in ('free', 'locked')
} {
    if { [empty_string_p $transition_key] } {
	incr workflow(place,$place_key,num_tokens)
    } else {
	incr workflow(transition,$transition_key,num_tokens)
    }
}

foreach place_key $workflow(places) {
    if { $workflow(place,$place_key,num_tokens) > 0 } {
	append workflow(place,$place_key,place_name) "\\n[string repeat "*" $workflow(place,$place_key,num_tokens)]"
	lappend workflow(selected_place_key) $place_key
    }
}
foreach transition_key $workflow(transitions) {
    if { $workflow(transition,$transition_key,num_tokens) > 0 } {
	append workflow(transition,$transition_key,transition_name) "\\n[string repeat "*" $workflow(transition,$transition_key,num_tokens)]"
	lappend workflow(selected_transition_key) $transition_key
    }
}



db_release_unused_handles

#####
#
# Create the workflow gif
#
#####

if { [wf_graphviz_installed_p] } {

    set dot_text [wf_generate_dot_representation workflow]
    
    set tmpfile [wf_graphviz_dot_exec -to_file -output gif $dot_text]
    
    set width_and_height ""
    if { ![catch { set image_size [ns_gifsize $tmpfile] } error] } {
	if { ![empty_string_p $image_size] } {
	    set width_and_height "width=[lindex $image_size 0] height=[lindex $image_size 1]"
	}
    }
    
    ad_set_client_property wf wf_net_tmpfile $tmpfile
    
    set workflow_img_tag "<img src=\"workflow-gif?[export_url_vars tmpfile]\" border=0 $width_and_height alt=\"Graphical representation of the process network\">"
} else {
    set workflow_img_tag ""
}


ad_return_template