ad_page_contract {} {
} -properties {
    task_list
}


if { ![info exist date_format] || [empty_string_p $date_format] } {
    set date_format "Mon fmDDfm, YYYY HH24:MI:SS"
}
if { ![info exists type] || [empty_string_p $type] } {
    set type enabled
}


if { ![string equal $type "enabled"] && ![string equal $type "own"] \
    && ![string equal $type "unassigned"] } {
    ad_return_error "Bad type" "Unrecognized type: Type can be 'enabled' or 'own'"
    ad_script_abort
}

set user_id [ad_get_user_id]

set select {
    {t.task_id} 
    {t.case_id} 
    {t.transition_key} 
    {t.enabled_date} 
    {to_char(t.enabled_date, :date_format) as enabled_date_pretty}
    {t.started_date}
    {to_char(t.started_date, :date_format) as started_date_pretty}
    {t.deadline}
    {to_char(t.deadline, :date_format) as deadline_pretty}
    {t.deadline - sysdate as days_till_deadline}
    {t.state} 
    {c.object_id} 
    {ot.object_type as object_type}
    {ot.pretty_name as object_type_pretty} 
    {acs_object.name(c.object_id) as object_name}
    {c.workflow_key}
    {wft.pretty_name as workflow_name}
    {sysdate}
    {t.estimated_minutes}
    {'' as task_url}
}
set from {
    {wf_cases c} 
    {acs_objects o} 
    {acs_object_types ot}
    {acs_object_types wft}
}
set where {
    {c.case_id = t.case_id} 
    {c.object_id = o.object_id} 
    {ot.object_type = o.object_type}
    {wft.object_type = c.workflow_key}
}

switch $type {
    unassigned {
        lappend select {tr.transition_name as task_name}
        lappend from {wf_tasks t}
        lappend from {wf_transitions tr}
        lappend from {wf_context_transition_info wcti}
        lappend where {tr.workflow_key = t.workflow_key}
        lappend where {tr.transition_key = t.transition_key}
        lappend where {not exists (select 1 from wf_task_assignments tas where tas.task_id = t.task_id)}
        lappend where {t.state = 'enabled'}
        lappend where {c.state = 'active'}
        lappend where {decode(wcti.access_privilege,'','t',acs_permission.permission_p(c.object_id, :user_id, wcti.access_privilege)) = 't'}
        lappend where {wcti.context_key = c.context_key}
        lappend where {wcti.workflow_key = tr.workflow_key}
        lappend where {wcti.transition_key = tr.transition_key}
    }
    default {
        lappend select {t.transition_name as task_name}
        lappend from {wf_user_tasks t}
        lappend where {t.user_id = :user_id} 
    }   
}

switch $type {
    own {
        lappend from {users u}
        lappend where {t.state = 'started'} {t.holding_user = t.user_id} {u.user_id = t.user_id}
    }
    enabled {
        lappend where {t.state = 'enabled'}
    }
}

set sql "
select [join $select ",\n       "]
from   [join $from ",\n       "]
where  [join $where "\n   and "]"

db_multirow task_list started_tasks_select $sql {
    set task_url "task?[export_vars -url {task_id}]"
}
