set caller_privilege [ttl_restrict_to_ttl_users]

if [string equal admin $caller_privilege] {
    set admin_url "[ad_conn package_url]admin"
} else {
    set admin_url ""
}

set package_id [ad_conn package_id]
set caller_id  [ad_conn user_id]

set master_template [ttl_master_template]

set navbar_list [ttl_navbar_list -nohome]

db_0or1row t {
    select 
        sort,
        mine_p,
        status,
        area_id,
        (select name from ttl_va_areas a
         where a.area_id = p.area_id) as area_name
    from ttl_prefs p
    where user_id = :caller_id
}

#
# 1) Whose tickets we're viewing
#

set mine_p [ttl_util_default mine_p f]

if [ttl_util_boolean mine_p] {
    set user_id_clause "
and exists (select 1 from ttl_assignments ta
            where ta.user_id = :caller_id
            and ta.ticket_id = t.ticket_id)"

} else {
    set user_id_clause ""
}

#
# 2) Which states we're viewing
#

set status [ttl_util_default status all]

if [string equal $status active] {
    set state_list {queued dev staging revision}
} elseif [string equal $status all] {
    set state_list {}
} else {
    set state_list $status
}

if ![empty_string_p $state_list] {
    set status_clause "\nand s.status in ('[join $state_list ',']')"
} else {
    set status_clause ""
}

#
# 3) How we're going to sort the results
#

set sort [ttl_util_default sort priority]

switch $sort {
    deadline {
        set order_clause "order by deadline asc, nvl(c.creation_date, t.last_modified) desc"
    } 
    activity {
        set order_clause "order by nvl(c.creation_date, t.last_modified) desc"
    }
    name {
        set order_clause "order by upper(ticket_name)"
    }
    area {
        set order_clause "order by lower(area_name) asc, priority asc, nvl(c.creation_date, t.last_modified) desc"
    } 
    status {
        set order_clause "order by lower(s.sort_key) asc, priority asc, nvl(c.creation_date, t.last_modified) desc"
    } 
    default {
        set order_clause "order by priority asc, nvl(c.creation_date, t.last_modified) desc"
    }
}

#
# 4) Which area we're viewing
#

set area_id [ttl_util_default area_id all]

if [string equal all $area_id] {
    set area_clause ""
} else {
    set area_clause "\nand t.area_id = :area_id"
}


# 5) Figure the page title

if [ttl_util_boolean mine_p] {
    set title "Your tickets"
} else {
    set title "Tickets"
}

if [exists_and_not_null area_name] {
    append title " in \"$area_name\""
}

db_multirow tickets t "
    select
        t.ticket_id,
        a.name as area_name,
        decode(t.priority, 1, 'High',
                           2, 'Medium',
                           3, 'Low',
                           t.priority) as ticket_priority,
        t.status,
        s.sort_key,
        t.name as ticket_name,
        ttl_relative_date(nvl(c.creation_date, t.last_modified)) as last_comment,
        decode(deadline, null, null, to_char(deadline, 'Mon DD')) as ticket_deadline
    from ttl_va_tickets t, ttl_status s, ttl_va_areas a, ttl_v_comments_latest c
    where a.context_id = :package_id  
    and a.area_id = t.area_id 
    and t.status = s.status
    and t.ticket_id = c.ticket_id(+) $user_id_clause $status_clause $area_clause
    $order_clause"

#
# For dimensional sliders
#

set name_slider     [ttl_pref_choices sort name Subject]
set status_slider   [ttl_pref_choices sort status Status]
set activity_slider [ttl_pref_choices sort activity "Last Activity"]
set area_slider     [ttl_pref_choices sort area "Area"]
set priority_slider [ttl_pref_choices sort priority Priority]
set deadline_slider [ttl_pref_choices sort deadline Deadline]

set ticket_owner_slider [ttl_pref_choices mine_p f All t Mine]
set ticket_state_slider [ttl_pref_choices status all all queued queued active active]

set area_optionlist [ttl_area_optionlist -all_option $area_id]
