--
-- packages/ticket-tracker/sql/ttracker-workflow-create.sql
--
-- 
-- @author Phong Nguyen (phong@arsdigita.com)
-- @author Tony Tseng (tony@arsidigta.com)
-- 
-- @creation-date 2000-11-15
--
-- @cvs-id $Id: ttracker-workflow-create.sql,v 1.1 2001/04/20 20:51:25 donb Exp $
--

-- This table will hold one row for each case using this workflow. 
create table wf_ttracker_cases (
    case_id    integer 
               constraint wf_ttkr_cases_pk
               primary key
	       constraint wf_ticket_cases_case_fk
	       references wf_cases
);


declare
    v_workflow_key      wf_workflows.workflow_key%TYPE;
begin
    v_workflow_key := workflow.create_workflow(
	workflow_key  => 'ttracker_wf', 
	pretty_name   => 'Ticket Tracker Process', 
	pretty_plural => 'Ticket Tracker Process', 
	description   => 'Workflow for processing a ticket in the ticket-tracker',
	table_name    => 'wf_ttracker_cases'
    );

    -- Creating places    
    workflow.add_place (
        workflow_key => 'ttracker_wf',
        place_key    => 'start',
        place_name   => 'Needs to be resolved',
        sort_order   => 1
    );    
    workflow.add_place (
        workflow_key => 'ttracker_wf',
        place_key    => 'to_be_clarified',
        place_name   => 'Needs to be clarified',
        sort_order   => 2
    );
    workflow.add_place (
        workflow_key => 'ttracker_wf',
        place_key    => 'to_be_verified',
        place_name   => 'Needs to be verified',
        sort_order   => 3
    );
    workflow.add_place (
        workflow_key => 'ttracker_wf',
        place_key    => 'end',
        place_name   => 'Closed',
        sort_order   => 4
    );

    -- Creating transitions
    workflow.add_transition (
        workflow_key    => 'ttracker_wf',
        transition_key  => 'resolve',
        transition_name => 'Resolve ticket',
        sort_order      => 1,
        trigger_type    => 'user'
    );
    workflow.add_transition (
        workflow_key    => 'ttracker_wf',
        transition_key  => 'clarify',
        transition_name => 'Clarify ticket description',
        sort_order      => 2,
        trigger_type    => 'user'
    );
    workflow.add_transition (
        workflow_key    => 'ttracker_wf',
        transition_key  => 'verify',
        transition_name => 'Verify result',
        sort_order      => 3,
        trigger_type    => 'user'
    );
end;
/
show errors

begin
    -- creating arcs
    -- resolve in
    -- in
    workflow.add_arc (
        workflow_key   => 'ttracker_wf',
        transition_key => 'resolve', 
        place_key      => 'start',
        direction      => 'in'
    );
    -- out
    workflow.add_arc (
        workflow_key      => 'ttracker_wf',
        transition_key    => 'resolve', 
        place_key         => 'to_be_clarified',
        direction         => 'out',
        guard_callback    => 'wf_callback.guard_attribute_true',
        guard_custom_arg  => 'need_clarification',
        guard_description => 'Description needs clarification'
    );
    workflow.add_arc (
        workflow_key      => 'ttracker_wf',
        transition_key    => 'resolve', 
        place_key         => 'to_be_verified',
        direction         => 'out',
        guard_callback    => '#',
        guard_description => 'Description clear; issue resolved'
    );
    
    -- verify
    -- in
    workflow.add_arc (
        workflow_key   => 'ttracker_wf',
        transition_key => 'verify', 
        place_key      => 'to_be_verified',
        direction      => 'in'
    );
    -- out
    workflow.add_arc (
        workflow_key      => 'ttracker_wf',
        transition_key    => 'verify', 
        place_key         => 'end',
        direction         => 'out',
        guard_callback    => 'wf_callback.guard_attribute_true',
        guard_custom_arg  => 'verified',
        guard_description => 'Result okay' 
    );
    workflow.add_arc (
        workflow_key      => 'ttracker_wf',
        transition_key    => 'verify', 
        place_key         => 'start',
        direction         => 'out',
        guard_callback    => '#',
        guard_custom_arg  => '',
        guard_description => 'Result not okay' 
    );

    -- clarify
    -- in
    workflow.add_arc (
        workflow_key   => 'ttracker_wf',
        transition_key => 'clarify', 
        place_key      => 'to_be_clarified',
        direction      => 'in'
    );
    -- out
    workflow.add_arc (
        workflow_key   => 'ttracker_wf',
        transition_key => 'clarify', 
        place_key      => 'start',
        direction      => 'out'
    );

end;
/
show errors

declare
    v_attribute_id acs_attributes.attribute_id%TYPE;
begin
    v_attribute_id := workflow.create_attribute(
	workflow_key   => 'ttracker_wf',
	attribute_name => 'need_clarification',
	datatype       => 'boolean',
	pretty_name    => 'Description needs clarification',
	default_value  => 'f'
    );

    insert into wf_transition_attribute_map
        (workflow_key, transition_key, attribute_id, sort_order) 
    values
        ('ttracker_wf', 'resolve', v_attribute_id, 1);

    v_attribute_id := workflow.create_attribute(
	workflow_key   => 'ttracker_wf',
	attribute_name => 'verified',
	datatype       => 'boolean',
	pretty_name    => 'Result is okay',
	default_value  => 't'
    );

    insert into wf_transition_attribute_map
        (workflow_key, transition_key, attribute_id, sort_order) 
    values
        ('ttracker_wf', 'verify', v_attribute_id, 1);
end;
/
show errors

insert into wf_context_transition_info (
     context_key, 
     workflow_key, 
     transition_key, 
     assignment_callback, 
     fire_callback, 
     notification_callback,
     notification_custom_arg,
     unassigned_callback,
     access_privilege
) values (
     'default', 
     'ttracker_wf', 
     'resolve', 
     'ttracker_callback.assign_task_to_assignee', 
     'ttracker_callback.resolve_fire', 
     'ttracker_callback.notification',
     'Resolve Ticket',
     'ttracker_callback.notify_admin',
     'admin'
);

insert into wf_context_transition_info (
     context_key, 
     workflow_key, 
     transition_key, 
     assignment_callback, 
     fire_callback, 
     notification_callback,
     notification_custom_arg,
     access_privilege
) values (
     'default', 
     'ttracker_wf', 
     'clarify', 
     'ttracker_callback.assign_task_to_submitter', 
     'ttracker_callback.clarify_fire', 
     'ttracker_callback.notification',
     'Clarify Description',
     'admin'
);

insert into wf_context_transition_info ( 
     context_key, 
     workflow_key, 
     transition_key, 
     assignment_callback, 
     notification_callback,
     notification_custom_arg,
     access_privilege
) values (
     'default', 
     'ttracker_wf', 
     'verify', 
     'ttracker_callback.assign_task_to_submitter', 
     'ttracker_callback.notification',
     'Verify Result',
     'admin'
);


insert into wf_context_task_panels (
    context_key, 
    workflow_key, 
    transition_key, 
    sort_key, 
    header, 
    template_url
) values (
    'default', 
    'ttracker_wf', 
    'resolve', 
    1, 
    'Ticket Info', 
    '/packages/ticket-tracker/wf-templates/ticket-info'
);

insert into wf_context_task_panels (
    context_key, 
    workflow_key, 
    transition_key, 
    sort_key, 
    header, 
    template_url
) values (
    'default', 
    'ttracker_wf', 
    'resolve', 
    2, 
    'What to do', 
    '/packages/ticket-tracker/wf-templates/resolve'
);

insert into wf_context_task_panels (
    context_key, 
    workflow_key, 
    transition_key, 
    sort_key, 
    header, 
    template_url
) values (
    'default', 
    'ttracker_wf', 
    'clarify', 
    1, 
    'Ticket Info', 
    '/packages/ticket-tracker/wf-templates/ticket-info'
);

insert into wf_context_task_panels (
    context_key, 
    workflow_key, 
    transition_key, 
    sort_key, 
    header, 
    template_url
) values (
    'default', 
    'ttracker_wf', 
    'clarify', 
    2, 
    'What to do', 
    '/packages/ticket-tracker/wf-templates/clarify'
);

insert into wf_context_task_panels (
    context_key, 
    workflow_key, 
    transition_key, 
    sort_key, 
    header, 
    template_url
) values (
    'default', 
    'ttracker_wf', 
    'verify', 
    1, 
    'Ticket Info', 
    '/packages/ticket-tracker/wf-templates/ticket-info'
);

insert into wf_context_task_panels (
    context_key, 
    workflow_key, 
    transition_key, 
    sort_key, 
    header, 
    template_url
) values (
    'default', 
    'ttracker_wf', 
    'verify', 
    2, 
    'What to do', 
    '/packages/ticket-tracker/wf-templates/verify'
);
      